<?php
/**
 * Perforce Swarm
 *
 * @copyright   2013-2017 Perforce Software. All rights reserved.
 * @license     Please see LICENSE.txt in top-level readme folder of this distribution.
 * @version     2017.1/1517929
 */

namespace Users\View\Helper;

use Zend\View\Helper\AbstractHelper;
use Notifications\Settings;

class NotificationSettings extends AbstractHelper
{
    const TITLE = "title";
    const ID    = "id";

    public static $userSettings = array(
        Settings::REVIEW_NEW => array (
            self::TITLE => 'A review is started (reviewers will be notified)',
            'settings'  => array (
                array (
                    self::ID    => Settings::IS_AUTHOR,
                    self::TITLE => 'If I\'m the author of the review',
                ),
                array (
                    self::ID    => Settings::IS_MEMBER,
                    self::TITLE => 'If I\'m a member of a project containing the branch',
                )
            )
        ),

        Settings::REVIEW_COMMENT_NEW => array (
            self::TITLE => 'Comments on a review are created ',
            'settings'  => array (
                array (
                    self::ID    => Settings::IS_AUTHOR,
                    self::TITLE => 'If I author the review',
                ),
                array (
                    self::ID    => Settings::IS_REVIEWER,
                    self::TITLE => 'If I\'m a reviewer on the review',
                ),
            )
        ),
        Settings::REVIEW_COMMENT_UPDATE => array (
            self::TITLE => 'Comments on a review are updated',
            'settings'  => array (
                array (
                    self::ID    => Settings::IS_AUTHOR,
                    self::TITLE => 'If I author the review',
                ),
                array (
                    self::ID    => Settings::IS_REVIEWER,
                    self::TITLE => 'If I\'m a reviewer on the review',
                ),
            )
        ),
        Settings::REVIEW_COMMENT_LIKED => array (
            'settings' => array (
                array (
                    self::ID    => Settings::IS_COMMENTER,
                    self::TITLE => 'Someone likes a comment that I wrote',
                )
            )
        ),

        Settings::REVIEW_FILES => array (
            self::TITLE => 'A review\'s files are updated',
            'settings'  => array (
                array (
                    self::ID    => Settings::IS_AUTHOR,
                    self::TITLE => 'If I author the review',
                ),
                array (
                    self::ID    => Settings::IS_REVIEWER,
                    self::TITLE => 'If I\'m a reviewer on the review',
                ),
            )
        ),
        Settings::REVIEW_VOTE => array (
            self::TITLE => 'A vote is cast on a review',
            'settings'  => array (
                array (
                    self::ID    => Settings::IS_AUTHOR,
                    self::TITLE => 'If I author the review',
                ),
                array (
                    self::ID    => Settings::IS_REVIEWER,
                    self::TITLE => 'If I\'m a reviewer on the review',
                )
            )
        ),
        Settings::REVIEW_STATE => array (
            self::TITLE => 'The state of a review changes (approve, archived, etc)',
            'settings'  => array (
                array (
                    self::ID    => Settings::IS_AUTHOR,
                    self::TITLE => 'If I author the review',
                ),
                array (
                    self::ID    => Settings::IS_REVIEWER,
                    self::TITLE => 'If I\'m a reviewer on the review',
                )
            )
        ),
        Settings::REVIEW_JOIN_LEAVE => array (
            'settings' => array (
                array (
                    self::ID    => Settings::IS_REVIEWER,
                    self::TITLE => 'Someone joins or leaves a review that I\'m a reviewer on',
                )
            )
        ),
        Settings::REVIEW_CHANGELIST_COMMIT => array (
            self::TITLE => 'A review or change is committed',
            'settings'  => array (
                array (
                    self::ID    => Settings::IS_REVIEWER,
                    self::TITLE => 'If I\'m a member of the review',
                ),
                array (
                    self::ID    => Settings::IS_MODERATOR,
                    self::TITLE => 'If I\'m a moderator on the branch (review)',
                ),
                array (
                    self::ID    => Settings::IS_MEMBER,
                    self::TITLE => 'If I\'m a member of a project containing the branch',
                )
            )
        ),
    );

    /**
     * Provides a table of setting for the user page current viewing.
     *
     * @param $settings     // get the settings passed in.
     * @return HTML Object  // return back the HTML object of the table
     */
    public function __invoke($settings)
    {
        return '<table id="notificationTable" class="table table-hover table-striped">'
        . $this->buildTableBody($this->getView(), $settings). '</table>';
    }

    /**
     * Build the table body based on user settings otherwise use default settings
     *
     * @param $view         To allow us to use the translate on the title we require the view.
     * @param $settings     Both the users and default settings are pass in
     * @return HTML Object  Return the table populated with User and default settings
     */
    public function buildTableBody($view, $settings)
    {
        // Empty out the body ready to be used.
        $body = '';

        // For each of the settings defined in this Class loop them and build the table.
        foreach (self::$userSettings as $key => $values) {
            // Group is the heading for all the indented options.
            $group = $settings[$key];
            // Check if the title value is set as some options are stand alone options.
            if (isset($values['title'])) {
                $body = $body . '<tr class="level1">'
                    . '<th colspan="2">' . $view->te($values['title']) . '</th>'
                    . '</tr>';
            }
            // for each of the options within this group build the rows for them.
            foreach ($values['settings'] as $options) {
                $settingsValues = $group[$options['id']];
                // Checks for settings to be applied.
                $notificationDisabled = $settingsValues['disabled'] === 'disabled'
                    ? ' class="notificationDisabled"' : '';
                // Incase option is disabled we set the option to the default value.
                $currentSetting = $settingsValues['disabled'] === 'disabled' ? $settingsValues['default']
                    : $settingsValues['value'];
                // Build the rows and populate with settings.
                $body = $body
                    . '<tr class="level2">'
                        .'<td'. $notificationDisabled . '>'
                            .  $view->te($options['title'])
                        . '</td>'
                        . '<td class="pull-right">'
                            . '<input id="' . $key . '_' . $options['id'] . '" '
                            . 'name="' . $key . '_' . $options['id'] . '" type="checkbox" data-default="'
                            . $settingsValues['default'] . '" ' . $currentSetting . ' ' . $settingsValues['disabled']
                            . ">"
                        . '</td>'
                    . '</tr>';
            }
        }
        return  '<tbody>' . $body . '</tbody>';
    }
}
